@@ -1,17 +1,26 @@
+from bisect import bisect_left
 import sys, os
 from PyQt6.QtCore import Qt, QModelIndex, QSettings, QUrl, QEvent
 from PyQt6.QtGui import QFontMetrics, QDesktopServices, QWindow
 from PyQt6.QtWidgets import *
-from .die import DIETableModel
-from .formats import read_dwarf, get_debug_sections, FormatError
-from .dwarfutil import has_code_location, ip_in_range
+
+from .die import DIETableModel, on_details_row_dclick
+from .formats import read_dwarf, get_debug_sections, load_companion_executable, FormatError, section_bytes, write_to_file
+from .dwarfutil import get_code_location, get_di_frames, has_code_location, ip_in_range, quote_filename, subprogram_name
 from .tree import DWARFTreeModel, cu_sort_key
 from .scriptdlg import ScriptDlg, make_execution_environment
-from .ui import setup_ui
-from .locals import LocalsDlg
+from .ui import setup_explorer, setup_ui
+from .locals import LocalsDlg, LoadedModuleDlgBase
+from .aranges import ArangesDlg
+from .frames import FramesDlg
+from .unwind import UnwindDlg
+from .funcmap import FuncMapDlg, GatherFuncsThread
+from .fx import WaitCursor, ArrowCursor
+from .treedlg import TreeDlg
 
 # Sync with version in setup.py
-version = (4, 0)
+version = (4, 55)
+the_app = None
 
 # TODO:
 # On MacOS, start without a main window, instead show the Open dialog
@@ -29,7 +38,8 @@ class DWARFParseError(Exception):
 
 class TheWindow(QMainWindow):
     def __init__(self):
-        QMainWindow.__init__(self)
+        super().__init__()
+        self.sett = None
         self.in_tree_nav = False
         self.font_metrics = QFontMetrics(QApplication.font())
 
@@ -38,6 +48,7 @@ class TheWindow(QMainWindow):
         self.setAcceptDrops(True)
 
         # The data model placeholders - to be populated once we read a file
+        self.dwarfinfo = None
         self.tree_model = None # Recreated between files
         self.die_model = None # Reused between DIEs
 
@@ -54,22 +65,31 @@ class TheWindow(QMainWindow):
                     print("The file contains no DWARF information, or it is in an unsupported format.")
             except Exception as exc:
                 print(format(exc))
+        elif os.environ.get("DWEX_LOADLAST") is not None and len(self.mru) > 0:
+            fa = self.mru[0]
+            if os.path.exists(fa[0]):
+                self.open_file(fa[0], fa[1:])
+
 
     def load_settings(self):
-        self.sett = QSettings('Seva', 'DWARFExplorer')
-        self.prefix = self.sett.value('General/Prefix', False, type=bool)
-        self.lowlevel = self.sett.value('General/LowLevel', False, type=bool)
-        self.hex = self.sett.value('General/Hex', False, type=bool)
-        self.sortcus = self.sett.value('General/SortCUs', True, type=bool)
-        self.sortdies = self.sett.value('General/SortDIEs', False, type=bool)
-        self.dwarfregnames = self.sett.value('General/DWARFRegNames', False, type=bool)
+        self.sett = sett = QSettings('Seva', 'DWARFExplorer')
+        self.prefix = sett.value('General/Prefix', False, type=bool)
+        self.lowlevel = sett.value('General/LowLevel', False, type=bool)
+        self.hex = sett.value('General/Hex', False, type=bool)
+        self.sortcus = sett.value('General/SortCUs', True, type=bool)
+        self.sortdies = sett.value('General/SortDIEs', False, type=bool)
+        self.dwarfregnames = sett.value('General/DWARFRegNames', False, type=bool)
         self.mru = []
         for i in range(0, 10):
-            f = self.sett.value("General/MRU%d" % i, False)
+            f = sett.value("General/MRU%d" % i, False)
             if f:
-                arch = self.sett.value("General/MRUArch%d" % i, None)
-                fa = (f,) if arch is None else (f,arch) 
-                self.mru.append(fa)        
+                arch = sett.value("General/MRUArch%d" % i, None)
+                fn = sett.value("General/MRUArchA%d" % i, None)
+                fa = (f,) if arch is None else (f, arch) if fn is None else (f, arch, fn)
+                self.mru.append(fa)
+        theme = sett.value("General/Theme", None, type=str)
+        if theme and theme in QStyleFactory.keys():
+            QApplication.setStyle(QStyleFactory.create(theme))
 
     ###################################################################
     # Done with init, now file stuff
@@ -77,44 +97,58 @@ class TheWindow(QMainWindow):
 
     # Callback for the Mach-O fat binary opening logic
     # Taking a cue from Hopper or IDA, we parse only one slice at a time
-    # Also reused for 
+    # arches is a list of strings in the simple case,
+    # list of strings and tuples in the tree case (Mach-O fat library)
     def resolve_arch(self, arches, title, message):
-        r = QInputDialog.getItem(self, title, message, arches, 0, False, Qt.WindowType.Dialog)
-        return arches.index(r[0]) if r[1] else None
+        with ArrowCursor():
+            if any(not isinstance(a, str) for a in arches):
+                dlg = TreeDlg(self, title, arches)
+                if dlg.exec() == QDialog.DialogCode.Accepted:
+                    mi = dlg.selection
+                    return mi[0] if len(mi) == 1 else mi
+            else:
+                r = QInputDialog.getItem(self, title, message, arches, 0, False, Qt.WindowType.Dialog)
+                return arches.index(r[0]) if r[1] else None
     
     # Can throw an exception
     # Returns None if it doesn't seem to contain DWARF
     # False if the user cancelled
     # True if the DWARF tree was loaded
-    def open_file(self, filename, arch = None):
-        self.start_wait()
-        try:
-            di = read_dwarf(filename, self.resolve_arch if arch is None else lambda arches, title, text: arches.index(arch))
+    def open_file(self, filename, slice = None):
+        with WaitCursor():
+            def recall_slice(slices, title, text):
+                if len(slice) == 1:
+                    return slices.index(slice[0])
+                else: # arch is a tuple, assuming no more than two levels
+                    (arch, fn) = slice
+                    (i, a) = next(ia for ia in enumerate(slices) if ia[1][0] == arch)
+                    j = a[1].index(fn)
+                    return (i, j)
+            di = read_dwarf(filename, self.resolve_arch if slice is None else recall_slice)
             if not di: # Covers both False and None
                 return di
             
-            return self.load_dwarfinfo(di, filename, arch)
-        finally:
-            self.end_wait()
+            return self.load_dwarfinfo(di, filename)
 
     # May throw if parsing fails
-    def load_dwarfinfo(self, di, filename, arch):
-        # Arch comes from opening with arch (via MRU), not from the interactive selection
+    def load_dwarfinfo(self, di, filename):
         # Some degree of graceful handling of wrong format
+        # File name in case of Mach-O bundles refers to the bundle path, not to the binary path within
         try:
-            slice = di._fat_arch if hasattr(di, '_fat_arch') else None
+            #TODO, slice
+            slice_code = di._slice_code if hasattr(di, '_slice_code') else None
             # Some cached top level stuff
             # Notably, iter_CUs doesn't cache (TODO, check that in the next version)
             di._ranges = None # Loaded on first use
             di._aranges = None
+            di._frames = None # Loaded on first use, False means missing
             def decorate_cu(cu, i):
                 cu._i = i
                 cu._lineprogram = None
                 cu._exprparser = None
                 return cu
             di._unsorted_CUs = [decorate_cu(cu, i) for (i, cu) in enumerate(di.iter_CUs())] # We'll need them first thing, might as well load here
-            if not len(di._unsorted_CUs):
-                return None # Weird, but saw it once - debug sections present, but no CUs
+
             # For quick CU search by offset within the info section, regardless of sorting
             di._CU_offsets = [cu.cu_offset for cu in di._unsorted_CUs]
             di._CUs = list(di._unsorted_CUs)
@@ -125,46 +159,62 @@ class TheWindow(QMainWindow):
                     cu._i = i
             di._locparser = None # Created on first use - but see #1683
 
+            if self.dwarfinfo is None:
+                setup_explorer(self)
             self.dwarfinfo = di
             self.filename = filename
-            self.tree_model = DWARFTreeModel(di, self.prefix, self.sortcus, self.sortdies)
-            self.the_tree.setModel(self.tree_model)
-            self.the_tree.selectionModel().currentChanged.connect(self.on_tree_selection)
+            has_CUs = bool(len(di._unsorted_CUs))
+            if has_CUs:
+                self.tree_model = DWARFTreeModel(di, self.prefix, self.sortcus, self.sortdies)
+                self.the_tree.setModel(self.tree_model)
+                self.the_tree.selectionModel().currentChanged.connect(self.on_tree_selection)
+            else: # Loading a binary with no CUs - possible
+                self.tree_model = None
+                self.the_tree.setModel(None)
+                self.die_table.setModel(None)
+                self.details_table.setModel(None)
             s = os.path.basename(filename)
-            if slice is not None:
-                s += ' (' + slice + ')'
+            if slice_code is not None:
+                s += ' (' + ':'.join(slice_code) + ')'
             self.setWindowTitle("DWARF Explorer - " + s)
+            # TODO: unite "enable on file load" into a collection
             self.savesection_menuitem.setEnabled(True)
-            self.switchslice_menuitem.setEnabled(slice is not None)
+            self.switchslice_menuitem.setEnabled(slice_code is not None)
+            self.loadexec_menuitem.setEnabled(di._format in (1, 5))
             self.back_menuitem.setEnabled(False)
             self.back_tbitem.setEnabled(False)
             self.forward_menuitem.setEnabled(False)
             self.forward_tbitem.setEnabled(False)
             self.followref_menuitem.setEnabled(False)
             self.followref_tbitem.setEnabled(False)
-            self.highlightcode_menuitem.setEnabled(True)
-            self.highlightsubstring_menuitem.setEnabled(True)
-            self.highlightcondition_menuitem.setEnabled(True)
-            self.highlightnothing_menuitem.setEnabled(True)
+            self.highlightcode_menuitem.setEnabled(has_CUs)
+            self.highlightsubstring_menuitem.setEnabled(has_CUs)
+            self.highlightcondition_menuitem.setEnabled(has_CUs)
+            self.highlightnothing_menuitem.setEnabled(has_CUs)
             self.copy_menuitem.setEnabled(False)
             self.copy_tbitem.setEnabled(False)
             self.copyline_menuitem.setEnabled(False)
             self.copytable_menuitem.setEnabled(False)
-            self.findbycondition_menuitem.setEnabled(True)
-            self.find_menuitem.setEnabled(True)
-            self.find_tbitem.setEnabled(True)
-            self.findip_menuitem.setEnabled(True)
-            self.byoffset_menuitem.setEnabled(True)
-            self.byoffset_tbitem.setEnabled(True)
-            self.localsat_menuitem.setEnabled(True)
+            self.findbycondition_menuitem.setEnabled(has_CUs)
+            self.find_menuitem.setEnabled(has_CUs)
+            self.find_tbitem.setEnabled(has_CUs)
+            self.findip_menuitem.setEnabled(has_CUs)
+            self.byoffset_menuitem.setEnabled(has_CUs)
+            self.byoffset_tbitem.setEnabled(has_CUs)
+            self.localsat_menuitem.setEnabled(has_CUs)
+            self.funcmap_menuitem.setEnabled(has_CUs)
+            self.aranges_menuitem.setEnabled(has_CUs)
+            self.frames_menuitem.setEnabled(True)
+            self.unwind_menuitem.setEnabled(di._format in (1, 5))
             self.on_highlight_nothing()
             # Navigation stack - empty
             self.navhistory = []
             self.navpos = -1
-            self.save_filename_in_mru(filename, slice)
-            LocalsDlg.reset(di)
+            self.save_filename_in_mru(filename, slice_code)
+            LoadedModuleDlgBase.reset(di)
+            LocalsDlg.reset()
             from .crash import set_binary_desc
-            set_binary_desc(("ELF", "MachO", "PE", "WASM", "ELFinA", "MachOinA")[di._format] + " " + di.config.machine_arch)
+            set_binary_desc(("ELF", "MachO", "PE", "WASM", "ELFinA", "MachOinA", "MachOinAinFat")[di._format] + " " + di.config.machine_arch)
             return True
         except AssertionError as ass: # Covers exeptions during parsing
             raise DWARFParseError(ass, di)        
@@ -172,15 +222,24 @@ class TheWindow(QMainWindow):
     def save_mru(self):
         for i, fa in enumerate(self.mru):
             self.sett.setValue("General/MRU%d" % i, fa[0])    
+
+            if len(fa) > 2:
+                self.sett.setValue("General/MRUArchA%d" % i, fa[2])
+            else:
+                self.sett.remove("General/MRUArchA%d" % i)
+
             if len(fa) > 1:
                 self.sett.setValue("General/MRUArch%d" % i, fa[1])
             else:
                 self.sett.remove("General/MRUArch%d" % i)
+
         for i in range(len(self.mru), 10):
             self.sett.remove("General/MRU%d" % i)
             self.sett.remove("General/MRUArch%d" % i)
+            self.sett.remove("General/MRUArchA%d" % i)
 
     # Open a file, display an error if failure
+    # Called from menu/File/Open, toolbar/Open, File/MRU, and the drop handler. MRU provides the arch
     def open_file_interactive(self, filename, arch = None):
         try:
             if self.open_file(filename, arch) is None:
@@ -188,11 +247,9 @@ class TheWindow(QMainWindow):
                     s = "The directory (bundle) could not be resolved to a DWARF containing file, or the file contains no DWARF information. Try navigating inside and open the executable file directly."
                 else:
                     s = "The file contains no DWARF information, or it is in an unsupported format."
-                QMessageBox(QMessageBox.Icon.Warning, "DWARF Explorer", s,
-                    QMessageBox.StandardButton.Ok, self).show()
+                self.show_warning(s)
         except FormatError as ferr:
-            QMessageBox(QMessageBox.Icon.Warning, "DWARF Explorer",
-                str(ferr), QMessageBox.StandardButton.Ok, self).show()
+            self.show_warning(str(ferr))
         except DWARFParseError as dperr:
             mb = QMessageBox(QMessageBox.Icon.Critical, "DWARF Explorer",
                 "Error parsing the DWARF information in this file. Would you like to save the debug section contents for manual analysis?",
@@ -229,8 +286,7 @@ class TheWindow(QMainWindow):
                         elif r == QMessageBox.StandardButton.No:
                             skip = True
                     if not skip:
-                        with open(section_file, 'wb') as f:
-                            f.write(section.stream.getbuffer())
+                        write_to_file(section_file, section_bytes(section))
                 except:
                     pass
 
@@ -241,33 +297,46 @@ class TheWindow(QMainWindow):
         if filename[0]:
             self.open_file_interactive(os.path.normpath(filename[0]))
 
+    def on_loadexec(self):
+        dir = os.path.dirname(self.mru[0][0]) if len(self.mru) > 0 else ''
+        filename = QFileDialog.getOpenFileName(self, None, dir)
+        if filename[0]:
+            try:
+                load_companion_executable(filename[0], self.dwarfinfo)
+            except FormatError as exc:
+                self.show_warning(str(exc))
+
     def populate_mru_menu(self):
         class MRUHandler(object):
-            def __init__(self, fa, win):
+            def __init__(self, fn, sc, win):
                 object.__init__(self)
-                self.fa = fa
+                self.fn = fn
+                self.sc = sc
                 self.win = win
             def __call__(self):
-                fa = self.fa
-                if os.path.exists(self.fa[0]):
-                    self.win.open_file_interactive(*fa)
+                if os.path.exists(self.fn):
+                    self.win.open_file_interactive(self.fn, self.sc)
                 else:
                     mb =  QMessageBox(QMessageBox.Icon.Critical, "DWARF Explorer",
-                        "The file or bundle %s does not exist or is not accessible. Shall we remove it from the recent file menu?" % (fa[0],),
+                        f"The file or bundle {self.fn} does not exist or is not accessible. Shall we remove it from the recent file menu?",
                         QMessageBox.StandardButton.Yes|QMessageBox.StandardButton.No, self.win)
                     mb.setEscapeButton(QMessageBox.StandardButton.No)
                     r = mb.exec()
                     if r == QMessageBox.StandardButton.Yes:
-                        self.win.delete_from_mru(fa)
+                        self.win.delete_from_mru((self.fn,) if self.sc is None else (self.fn,) + self.sc)
 
-        for i, fa in enumerate(self.mru):
-            s = fa[0]
-            if len(fa) > 1:
-                s += ' (' + fa[1] + ')'
-            self.mru_menu.addAction(s).triggered.connect(MRUHandler(fa, self))
+        for i, fnsc in enumerate(self.mru):
+            s = fn = fnsc[0]
+            if len(fnsc) > 1:
+                slice_code = fnsc[1:]
+                s += ' (' + ':'.join(slice_code) + ')'
+            else:
+                slice_code = None
+            self.mru_menu.addAction(s).triggered.connect(MRUHandler(fn, slice_code, self))
 
-    def save_filename_in_mru(self, filename, arch = None):
-        mru_record = (filename,) if arch is None else (filename, arch)
+    # slice_code is a tuple
+    def save_filename_in_mru(self, filename, slice_code = None):
+        mru_record = (filename,) if slice_code is None else (filename,) + slice_code
         try:
             i = self.mru.index(mru_record)
         except ValueError:
@@ -314,10 +383,7 @@ class TheWindow(QMainWindow):
             r = QFileDialog.getSaveFileName(self, "Save a section", self.filename + '.' + section_name)
             if r[0]:
                 try:
-                    section = sections[section_name]
-                    with open(r[0], 'wb') as f:
-                        # Assumes the section is a BytesIO - implementation dependent
-                        f.write(section.stream.getbuffer())
+                    write_to_file(r[0], section_bytes(sections[section_name]))
                 except Exception as exc:
                     QMessageBox(QMessageBox.Icon.Critical, "DWARF Explorer",
                         "Error saving the section data:\n\n" + format(exc),
@@ -334,28 +400,29 @@ class TheWindow(QMainWindow):
 
     # Index is a tree index - the DIE is the data object within
     def display_die(self, index):
-        die = index.internalPointer()
-        die_table = self.die_table
-        if not self.die_model:
-            self.die_model = DIETableModel(die, self.prefix, self.lowlevel, self.hex, self.dwarfregnames)
-            die_table.setModel(self.die_model)
-            die_table.selectionModel().currentChanged.connect(self.on_attribute_selection)
-        else:
-            self.die_model.display_DIE(die)
-        self.die_table.resizeColumnsToContents()
-        self.details_table.setModel(None)
-        self.followref_menuitem.setEnabled(False)
-        self.followref_tbitem.setEnabled(False)
-        self.cuproperties_menuitem.setEnabled(True)
-        self.die_table.setCurrentIndex(QModelIndex()) # Will cause on_attribute_selection
+        if self.details_table and self.die_table: # Short out for #1753
+            die = index.internalPointer()
+            die_table = self.die_table
+            if not self.die_model:
+                self.die_model = DIETableModel(die, self.prefix, self.lowlevel, self.hex, self.dwarfregnames)
+                die_table.setModel(self.die_model)
+                die_table.selectionModel().currentChanged.connect(self.on_attribute_selection)
+            else:
+                self.die_model.display_DIE(die)
+            self.die_table.resizeColumnsToContents()
+            self.details_table.setModel(None)
+            self.followref_menuitem.setEnabled(False)
+            self.followref_tbitem.setEnabled(False)
+            self.cuproperties_menuitem.setEnabled(True)
+            self.die_table.setCurrentIndex(QModelIndex()) # Will cause on_attribute_selection
 
-        #TODO: resize the attribute table vertically dynamically
-        #attr_count = self.die_model.rowCount(None)
-        #die_table.resize(die_table.size().width(),
-        #    die_table.rowViewportPosition(attr_count-1) + 
-        #        die_table.rowHeight(attr_count-1) +
-        #        die_table.horizontalHeader().size().height() + 1 + attr_count)
-        #self.rpane_layout.update()
+            #TODO: resize the attribute table vertically dynamically
+            #attr_count = self.die_model.rowCount(None)
+            #die_table.resize(die_table.size().width(),
+            #    die_table.rowViewportPosition(attr_count-1) + 
+            #        die_table.rowHeight(attr_count-1) +
+            #        die_table.horizontalHeader().size().height() + 1 + attr_count)
+            #self.rpane_layout.update()
 
     # Invoked for tree clicks and keyboard navigation, ref follow, back-forward
     def on_tree_selection(self, index, prev = None):
@@ -377,8 +444,9 @@ class TheWindow(QMainWindow):
             self.details_table.setModel(details_model)
             if details_model is not None:
                 self.details_table.resizeColumnsToContents()
-                self.details_warning.setVisible(details_model.warning is not None)
-                if details_model.warning is not None:
+                has_warning = hasattr(details_model, 'warning') and details_model.warning is not None
+                self.details_warning.setVisible(has_warning)
+                if has_warning:
                     self.details_warning.setText(details_model.warning)
             else:
                 self.details_warning.setVisible(False)
@@ -420,30 +488,23 @@ class TheWindow(QMainWindow):
             self.forward_tbitem.setEnabled(np > 0)
 
     def followref(self, index = None):
-        self.start_wait() # TODO: only show the wait cursor if it's indeed time consuming
-        if index is None:
-            index = self.die_table.currentIndex()
-        navitem = self.die_model.ref_target(index)  # Retrieve the ref target from the DIE model...
-        if navitem:
-            target_tree_index = self.tree_model.index_for_navitem(navitem) # ...and feed it to the tree model.
-            if target_tree_index:
-                self.the_tree.setCurrentIndex(target_tree_index) # Calls on_tree_selection internally
-        self.end_wait()
+        with WaitCursor():
+            # TODO: only show the wait cursor if it's indeed time consuming
+            if index is None:
+                index = self.die_table.currentIndex()
+            navitem = self.die_model.ref_target(index)  # Retrieve the ref target from the DIE model...
+            if navitem:
+                target_tree_index = self.tree_model.index_for_navitem(navitem) # ...and feed it to the tree model.
+                if target_tree_index:
+                    self.the_tree.setCurrentIndex(target_tree_index) # Calls on_tree_selection internally
 
     # Called for double-click on a reference type attribute, and via the menu
     def on_followref(self):
         self.followref()
 
-    # Back-forward mouse buttons are shortcuts for back/forward navigation
-    # Qt docs claim capturing is not necessary
-    #def mouseReleaseEvent(self, evt):
-    #    QMainWindow.mouseReleaseEvent(self, evt)
-    #    b = evt.button()
-    #    if b == Qt.MouseButton.BackButton:
-    #        self.on_nav(1)
-    #    elif b == Qt.MouseButton.ForwardButton:
-    #        self.on_nav(-1)
-        
+    def on_details_dclick(self, index):
+        if index.isValid():
+            on_details_row_dclick(index, index.internalPointer(), self)
 
     ##########################################################################
     # Find/Find next stuff
@@ -508,8 +569,7 @@ class TheWindow(QMainWindow):
                 if index:
                     self.the_tree.setCurrentIndex(index)
                 else:
-                    QMessageBox(QMessageBox.Icon.Warning, "DwARF Explorer", "The specified offset was not found. It could be beyond the section size, or fall into a CU header area.",
-                        QMessageBox.StandardButton.Ok, self)
+                    self.show_warning("The specified offset was not found. It could be beyond the section size, or fall into a CU header area.")
             except ValueError:
                 pass
 
@@ -530,33 +590,50 @@ class TheWindow(QMainWindow):
         if index:
             self.the_tree.setCurrentIndex(index)
 
+    def on_changetheme(self):
+        themes = ["Default",] + QStyleFactory.keys()
+        theme = self.sett.value('General/Theme', None, type=str)
+        theme_no = themes.index(theme) if theme and theme in themes else 0
+        r = QInputDialog.getItem(self, "Theme", "Please select the visual theme:", themes, theme_no, False, Qt.WindowType.Dialog)
+        if r[1]:
+            new_theme_no = themes.index(r[0])
+            if new_theme_no == 0 and theme:
+                self.sett.remove('General/Theme')
+            elif new_theme_no > 0:
+                self.sett.setValue('General/Theme', themes[new_theme_no])
+
+            if new_theme_no > 0:
+                QApplication.setStyle(QStyleFactory.create(themes[new_theme_no]))
+            else:
+                QApplication.setStyle(None)
+    
     ##########################################################################
     ##########################################################################
 
     def on_about(self):
-        QMessageBox(QMessageBox.Icon.Information, "About...", "DWARF Explorer v." + '.'.join(str(v) for v in version) + "\n\nSeva Alekseyev, 2020-2023\nsevaa@sprynet.com\n\ngithub.com/sevaa/dwex",
+        QMessageBox(QMessageBox.Icon.Information, "About...", "DWARF Explorer v." + '.'.join(str(v) for v in version) + "\n\nSeva Alekseyev, 2020-2024\nsevaa@sprynet.com\n\ngithub.com/sevaa/dwex",
             QMessageBox.StandardButton.Ok, self).show()
 
     def on_updatecheck(self):
         from urllib.request import urlopen
         import json
         try:
-            self.start_wait()
-            resp = urlopen('https://api.github.com/repos/sevaa/dwex/releases')
-            if resp.getcode() == 200:
-                releases = resp.read()
-                self.end_wait()
-                releases = json.loads(releases)
-                if len(releases) > 0:
-                    max_ver = max(tuple(int(v) for v in r['tag_name'].split('.')) for r in releases)
-                    max_tag = '.'.join(str(i) for i in max_ver)
-                    if max_ver > version:
-                        s = "DWARF Explorer v." + max_tag + " is out. Use \"pip install --upgrade dwex\" to update."
-                    else: 
-                        s = "You have the latest version."
-                    QMessageBox(QMessageBox.Icon.Information, "DWARF Explorer", s, QMessageBox.StandardButton.Ok, self).show()
+            releases = False
+            with WaitCursor():
+                resp = urlopen('https://api.github.com/repos/sevaa/dwex/releases')
+                if resp.getcode() == 200:
+                    releases = json.loads(resp.read())
+            if releases and len(releases) > 0:
+                max_ver = max(tuple(int(v) for v in r['tag_name'].split('.')) for r in releases)
+                max_tag = '.'.join(str(i) for i in max_ver)
+                if max_ver > version:
+                    s = "DWARF Explorer v." + max_tag + " is out. Use \"pip install --upgrade dwex\" to update."
+                    # TODO: not only pip
+                else: 
+                    s = "You have the latest version."
+                QMessageBox(QMessageBox.Icon.Information, "DWARF Explorer", s, QMessageBox.StandardButton.Ok, self).show()
         except:
-            self.end_wait()
+            pass
 
     def on_exit(self):
         self.destroy()
@@ -675,7 +752,8 @@ class TheWindow(QMainWindow):
         self.highlightsubstring_menuitem.setChecked(False)
         self.highlightcondition_menuitem.setChecked(False)
         self.manage_hlnavigation(False)
-        self.tree_model.clear_highlight()
+        if self.tree_model:
+            self.tree_model.clear_highlight()
 
     def on_nexthl(self):
         index = self.tree_model.find(self.the_tree.currentIndex(), self.tree_model.is_highlighted, False)
@@ -707,7 +785,7 @@ class TheWindow(QMainWindow):
         cb.setText(v)
 
     def on_copyvalue(self):
-        t = self.details_table if self.details_table.hasFocus() else self.die_table
+        t = self.details_table if self.details_table.hasFocus() and self.details_table.model() else self.die_table
         m = t.model()
         self.on_copy(m.data(t.currentIndex(), Qt.ItemDataRole.DisplayRole) or "")
 
@@ -715,7 +793,7 @@ class TheWindow(QMainWindow):
         t = self.details_table if self.details_table.hasFocus() else self.die_table
         m = t.model()
         row = t.currentIndex().row()
-        line = "\t".join(m.data(m.index(row, c, QModelIndex()), Qt.ItemDataRole.DisplayRole) or ""
+        line = "\t".join(str(m.data(m.index(row, c, QModelIndex()), Qt.ItemDataRole.DisplayRole) or "")
             for c in range(0, m.columnCount(QModelIndex())))
         self.on_copy(line)
 
@@ -723,16 +801,79 @@ class TheWindow(QMainWindow):
         t = self.details_table if self.details_table.hasFocus() else self.die_table
         m = t.model()
         table_text = "\n".join(
-                "\t".join(m.data(m.index(r, c, QModelIndex()), Qt.ItemDataRole.DisplayRole)  or ""
+                "\t".join(str(m.data(m.index(r, c, QModelIndex()), Qt.ItemDataRole.DisplayRole)  or "")
                 for c in range(0, m.columnCount(QModelIndex())))
             for r in range(0, m.rowCount(QModelIndex())))
         self.on_copy(table_text)
 
+    ##################################################################
+
     def on_localsat(self):
         dlg = LocalsDlg(self, self.dwarfinfo, self.prefix, self.dwarfregnames, self.hex)
         if dlg.exec() == QDialog.DialogCode.Accepted and dlg.selected_die:
              self.the_tree.setCurrentIndex(self.tree_model.index_for_die(dlg.selected_die))
 
+    def on_funcmap(self):
+        th = GatherFuncsThread(self, self.dwarfinfo)
+        def done():
+            if not pd.wasCanceled():
+                pd.close()
+
+            if th.funcs:
+                dlg = FuncMapDlg(self, self.hex, th.funcs)
+                if dlg.exec() == QDialog.DialogCode.Accepted and dlg.selected_die:
+                    self.the_tree.setCurrentIndex(self.tree_model.index_for_die(dlg.selected_die))
+            elif th.exc:
+                print(th.exc)
+
+        last_CU = self.dwarfinfo._unsorted_CUs[-1]
+        pd = QProgressDialog("Gathering functions...", "Cancel", 0, last_CU.cu_offset + last_CU.size, self, Qt.WindowType.Dialog)
+        pd.canceled.connect(th.cancel)
+        pd.show()
+        th.progress.connect(pd.setValue)
+        th.finished.connect(done)
+        th.start() # Will continue in done
+
+    def on_aranges(self):
+        from elftools.common.exceptions import ELFParseError
+        try:
+            ara = self.dwarfinfo.get_aranges()
+        except ELFParseError: # Catching the IAR < 9.30 aranges misalignment issue
+            self.show_warning("The aranges section in this binary is corrupt.")
+            return
+
+        if ara:
+            dlg = ArangesDlg(self, ara, self.dwarfinfo, self.hex)
+            if dlg.exec() == QDialog.DialogCode.Accepted and dlg.selected_cu_offset is not None:
+                di = self.dwarfinfo
+                i = bisect_left(di._CU_offsets, dlg.selected_cu_offset)
+                if i < len(di._CU_offsets) and di._CU_offsets[i] == dlg.selected_cu_offset:
+                    die = di._unsorted_CUs[i].get_top_DIE()
+                    self.the_tree.setCurrentIndex(self.tree_model.index_for_die(die))
+        else:
+            self.show_warning("This binary does not have an aranges section.")
+            
+    def on_frames(self):
+        try:
+            entries = get_di_frames(self.dwarfinfo)
+            if entries:
+                FramesDlg(self, entries, self.dwarfinfo, self.dwarfregnames, self.hex).exec()
+                # TODO: navigate to function
+            else:
+                self.show_warning("This binary does not have neither an eh_frames section nor a debug_frames section.")
+        except KeyError: # 1761
+            self.show_warning("Error parsing the frames section in this binary. Please report to the tech support: menu/Help/Report an issue.")
+
+            
+    def on_unwind(self):
+        if self.dwarfinfo._unwind_sec:
+            UnwindDlg(self, self.dwarfinfo._unwind_sec, self.dwarfinfo, self.dwarfregnames, self.hex).exec()
+            # TODO: navigate to function
+        elif self.dwarfinfo._has_exec:
+            self.show_warning("Neither this binary/slice nor the companion executable has an unwind_info section.")
+        else: # TODO: distinguish .o files where the section is named differently
+            self.show_warning("This binary/slice does not have an unwind_info section, but the corresponding executable might. Use File/Load companion... to find and load one.")
+
     # If the details pane has data - reload that
     def refresh_details(self):
         index = self.die_table.currentIndex()
@@ -751,54 +892,13 @@ class TheWindow(QMainWindow):
 
     # All purpose debug hook
     def on_debug(self):
-        self.dwarfinfo.CFI_entries()
-        return
+        pass
 
+    def show_warning(self, s):
+        QMessageBox(QMessageBox.Icon.Warning, "DWARF Explorer", s, QMessageBox.StandardButton.Ok, self).show()
 
-        import io
-        from elftools.dwarf.dwarfinfo import DWARFInfo, DwarfConfig, DebugSectionDescriptor
-        # Read the three saved sections as bytestreams
-        base = os.environ.get("DWEX_ARG")
-        with open(base + '.info.dat', 'rb') as f:
-            info = f.read()
-        with open(base + '.abbrev.dat', 'rb') as f:
-            abbrev = f.read()
-        with open(base + '.str.dat', 'rb') as f:
-            str = f.read()
-
-        # Parse the DWARF info
-        di = DWARFInfo(
-            config = DwarfConfig(little_endian = True, default_address_size = 8, machine_arch = "ARM64"),
-            debug_info_sec = DebugSectionDescriptor(io.BytesIO(info), '__debug_info', None, len(info), 0),
-            debug_aranges_sec = None,
-            debug_abbrev_sec = DebugSectionDescriptor(io.BytesIO(abbrev), '__debug_abbrev', None, len(abbrev), 0),
-            debug_frame_sec = None,
-            eh_frame_sec = None,
-            debug_str_sec = DebugSectionDescriptor(io.BytesIO(str), '__debug_str', None, len(str), 0),
-            debug_loc_sec = None,
-            debug_ranges_sec = None,
-            debug_line_sec = DebugSectionDescriptor(io.BytesIO(b''), '__debug_line', None, 0, 0),
-            debug_pubtypes_sec = None,
-            debug_pubnames_sec = None,
-            debug_addr_sec=None,
-            debug_str_offsets_sec=None,
-            debug_line_str_sec=None,
-            debug_loclists_sec = None,
-            debug_rnglists_sec = None,
-            debug_sup_sec = None,
-            gnu_debugaltlink_sec = None
-        )
-        di._start_address = 0
-        di._format = 1
-
-        self.load_dwarfinfo(di, "", "ARM64")
-
-    # Doesn't quite work for the delay on tree expansion :( TODO: timer checks before lighting up this
-    def start_wait(self):
-        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
-
-    def end_wait(self):
-        QApplication.restoreOverrideCursor()
+    def expr_formatter(self):
+        return self.die_model.expr_formatter
 
 def on_exception(exctype, exc, tb):
     if isinstance(exc, MemoryError):
@@ -811,6 +911,12 @@ def on_exception(exctype, exc, tb):
     elif isinstance(exc, Exception):
         from .crash import report_crash
         report_crash(exc, exc.__traceback__, version)
+        try:
+            global the_app
+            if the_app and the_app.win and the_app.win.sett:
+                the_app.win.sett.setValue("Crashed", True)
+        except Exception:
+            pass
         sys.excepthook = on_exception.prev_exchook
         sys.exit(1)
     elif on_exception.prev_exchook:
@@ -818,7 +924,8 @@ def on_exception(exctype, exc, tb):
 
 class TheApp(QApplication):
     def __init__(self):
-        QApplication.__init__(self, [])
+        super().__init__([])
+        self.win = None
 
     def notify(self, o, evt):
         if evt.type() == QEvent.Type.MouseButtonPress and isinstance(o, QWindow) and hasattr(evt, "button"):
@@ -834,7 +941,7 @@ class TheApp(QApplication):
         self.exec()
 
 def main():
-    under_debugger = sys.gettrace() # Lame way to detect a debugger
+    under_debugger = hasattr(sys, 'gettrace') and sys.gettrace() or hasattr(sys, 'monitoring') and sys.monitoring.get_tool(sys.monitoring.DEBUGGER_ID) # Lame way to detect a debugger
     if not under_debugger: 
         on_exception.prev_exchook = sys.excepthook
         sys.excepthook = on_exception
@@ -842,8 +949,9 @@ def main():
     from .patch import monkeypatch
     monkeypatch()
 
-    TheApp().start()
-            
+    global the_app
+    the_app = TheApp()
+    the_app.start()
 
 # For running via "python -m dwex"
 # Running this file directly won't work, it relies on being in a module
