@@ -1,15 +1,17 @@
 from bisect import bisect_right
+from collections.abc import Sequence
 from PyQt6.QtCore import Qt, QAbstractTableModel, QModelIndex
-from PyQt6.QtGui import QBrush
 from elftools.dwarf.locationlists import LocationParser, LocationExpr
-from elftools.dwarf.dwarf_expr import DWARFExprParser
+from elftools.dwarf.dwarf_expr import DWARFExprParser, DWARFExprOp
 from elftools.dwarf.descriptions import _DESCR_DW_LANG, _DESCR_DW_ATE, _DESCR_DW_ACCESS, _DESCR_DW_INL, _DESCR_DW_CC
 from elftools.common.exceptions import ELFParseError
 
-from dwex.exprutil import ExprFormatter
+from .exprutil import ExprFormatter, is_parsed_expression
 from .dwarfone import DWARFExprParserV1
 from .dwarfutil import *
 from .details import GenericTableModel, FixedWidthTableModel
+from .exprdlg import ExpressionTableModel, ExpressionDlg, op_has_nested_expression
+from .fx import blue_brush, ltgrey_brush
 
 MAX_INLINE_BYTEARRAY_LEN = 32
 
@@ -17,13 +19,14 @@ def is_long_blob(attr):
     val = attr.value
     return ((isinstance(val, bytes) and attr.form not in ('DW_FORM_strp', 'DW_FORM_string')) or is_int_list(val)) and len(val) > MAX_INLINE_BYTEARRAY_LEN
 
+class ExprParseError(Exception):
+    expr = None
+    opcode = None
+
 #------------------------------------------------
 # DIE formatter
 #------------------------------------------------
 
-_blue_brush = QBrush(Qt.GlobalColor.blue)
-_ltgrey_brush = QBrush(Qt.GlobalColor.lightGray)
-
 _ll_headers = ("Attribute", "Offset", "Form", "Raw", "Value")
 _noll_headers = ("Attribute", "Form", "Value")
 _meta_desc = ('DIE offset', 'DIE size', 'Abbrev code', 'Has children') # Anything else?
@@ -42,9 +45,10 @@ class DIETableModel(QAbstractTableModel):
         self.headers = _ll_headers if self.lowlevel else _noll_headers
         self.meta_count = _meta_count if lowlevel else 0
         self.expr_formatter = ExprFormatter(regnames, prefix, die.dwarfinfo.config.machine_arch, die.cu['version'], hex)
+        self.expr_formatter.cfa_resolver = self.resolve_cfa
 
     from .ranges import show_ranges
-    from .locs import parse_location, show_location 
+    from .locs import parse_location, show_location, show_loclist, resolve_cfa
 
     def headerData(self, section, ori, role):
         if ori == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
@@ -80,7 +84,7 @@ class DIETableModel(QAbstractTableModel):
             col = index.column()
             if col == 0:
                 # Unknown keys come across as ints
-                return key if self.prefix or not str(key).startswith('DW_AT_') else key[6:]
+                return self.format_attr_name(key)
             elif col == 1:
                 return hex(attr.offset) if self.lowlevel else self.format_form(attr.form)
             elif col == 2:
@@ -90,18 +94,26 @@ class DIETableModel(QAbstractTableModel):
             elif col == 4:
                 return self.format_value(attr)
         elif role == Qt.ItemDataRole.ToolTipRole:
+            tip = ''
             if attr.form in ('DW_FORM_ref', 'DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8', 'DW_FORM_ref_addr'):
-                return "Double-click to follow"
+                rt = self.ref_target_from_attr(attr)
+                if rt: 
+                    (target_cu, target_offset) = rt
+                    target = target_cu.get_DIE_from_refaddr(target_offset)
+                    if target and not target.is_null():
+                        tip = self.format_tag(target.tag) + ' ' + safe_DIE_name(target, '(unknown)') + '\n'
+                tip += "Double-click to follow"
             elif attr.form in ('DW_FORM_ref_sig8', 'DW_FORM_ref_sup4', 'DW_FORM_ref_sup8'):
-                return "Unsupported reference format"
+                tip += "Unsupported reference format"
             elif is_long_blob(attr):
-                return "Click to see it all"
+                tip += "Click to see it all"
+            return tip
         elif role == Qt.ItemDataRole.ForegroundRole:
             if attr.form in ('DW_FORM_ref', 'DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8', 'DW_FORM_ref_addr'):
-                return _blue_brush
+                return blue_brush
         elif role == Qt.ItemDataRole.BackgroundRole:
             if self.lowlevel and index.column() == 3 and attr.raw_value == attr.value:
-                return _ltgrey_brush                
+                return ltgrey_brush                
 
     # Data for the metadata lines - ones that are not attributes
     def meta_data(self, index, role):
@@ -120,20 +132,42 @@ class DIETableModel(QAbstractTableModel):
                 elif row == 3:
                     return str(self.die.has_children)
         elif role == Qt.ItemDataRole.BackgroundRole:
-            return _ltgrey_brush
+            return ltgrey_brush
+        
+    def parse_expr(self, expr):
+        """expr is a blob
+        """
+        if self.die.cu._exprparser is None:
+            self.die.cu._exprparser = DWARFExprParser(self.die.cu.structs) if self.die.cu['version'] > 1 else DWARFExprParserV1(self.die.cu.structs)
+        try:
+            return self.die.cu._exprparser.parse_expr(expr)
+        except KeyError as ke:
+            pe = ExprParseError()
+            pe.expr = expr
+            pe.opcode = ke.args[0]
+            raise pe
+    
+    def format_op(self, op):
+        return self.expr_formatter.format_op(*op)
+    
+    def format_expr(self, expr, len_cutoff = None):
+        """expr is a list of parsed operations
+        """
+        if len_cutoff is not None and len(expr) > len_cutoff:
+            return '; '.join(self.format_op(op) for op in expr[:len_cutoff]) + f'...+{len(expr)-len_cutoff} more'
+        else:
+            return '; '.join(self.format_op(op) for op in expr)
 
     # Expr is an expression blob
     # Returns a list of strings for ops
     # Format: op arg, arg...
-    def dump_expr(self, expr):
-        if self.die.cu._exprparser is None:
-            self.die.cu._exprparser = DWARFExprParser(self.die.cu.structs) if self.die.cu['version'] > 1 else DWARFExprParserV1(self.die.cu.structs)
-
+    def dump_expr(self, expr, len_cutoff = None):
+        """expr is a blob"""
         # Challenge: for nested expressions, args is a list with a list of commands
         # For those, the format is: op {op arg, arg; op arg, arg}
         # Can't just check for iterable, str is iterable too
-        return [self.expr_formatter.format_op(*op) for op in self.die.cu._exprparser.parse_expr(expr)]
-
+        return self.format_expr(self.parse_expr(expr), len_cutoff)
+    
     # Big DIE attribute value interpreter for the top right table
     def format_value(self, attr):
         try:
@@ -158,7 +192,7 @@ class DIETableModel(QAbstractTableModel):
                 if ll is None:
                     return "(parse error - please report at github.com/sevaa/dwex)"
                 elif isinstance(ll, LocationExpr):
-                    return '; '.join(self.dump_expr(ll.loc_expr))
+                    return self.dump_expr(ll.loc_expr)
                 else:
                     return "Loc list: 0x%x" % attr.value
             elif key == 'DW_AT_language':
@@ -189,7 +223,7 @@ class DIETableModel(QAbstractTableModel):
             elif key == 'DW_AT_stmt_list':
                 return 'LNP at 0x%x' % val
             elif key in ('DW_AT_upper_bound', 'DW_AT_lower_bound') and is_block(form):
-                return '; '.join(self.dump_expr(val))
+                return self.dump_expr(val)
             elif isinstance(val, bytes):
                 if form in ('DW_FORM_strp', 'DW_FORM_string', 'DW_FORM_line_strp', 'DW_FORM_strp_sup',
                     'DW_FORM_strx', 'DW_FORM_strx1', 'DW_FORM_strx2', 'DW_FORM_strx3', 'DW_FORM_strx4'):
@@ -213,6 +247,17 @@ class DIETableModel(QAbstractTableModel):
             else:
                 return hex(val) if self.hex and isinstance(val, int) else str(val)
         # except BaseException as exc:
+        except ExprParseError as exc: # Shorting out #1740
+            from .__main__ import version
+            from .crash import report_crash
+            from inspect import currentframe
+            ctxt = {'attr': attr,
+                        'die': die,
+                        'dwarf_version': dwarf_version,
+                        'opcode': exc.opcode,
+                        'expr': ' '.join(f'{b:02X}' for b in exc.expr)}
+            report_crash(exc, exc.__traceback__, version, currentframe(), ctxt)
+            return f"(unknown expression opcode 0x{exc.opcode:02X})"
         except ELFParseError as exc:
             import sys
             if not sys.gettrace():
@@ -353,7 +398,7 @@ class DIETableModel(QAbstractTableModel):
                 # TODO: commands vs states
                 return GenericTableModel(('Address', 'File', 'Line', 'Stmt', 'Basic block', 'End seq', 'End prologue', 'Begin epilogue'), states)
             elif key in ('DW_AT_upper_bound', 'DW_AT_lower_bound') and is_block(form):
-                return GenericTableModel(("Command",), [(o,) for o in self.dump_expr(attr.value)])
+                return ExpressionTableModel(self.parse_expr(attr.value), self.expr_formatter)
             elif is_long_blob(attr):
                 val = attr.value
                 def format_line(off):
@@ -362,6 +407,17 @@ class DIETableModel(QAbstractTableModel):
                 lines = [format_line(off) for off in range(0, len(val), MAX_INLINE_BYTEARRAY_LEN)]
                 return FixedWidthTableModel(('Offset (%s)' % ('hex' if self.hex else 'dec'), 'Bytes'), lines)
         return None
+    
+    def ref_target_from_attr(self, attr):
+        val = attr.value
+        form = attr.form
+        if form in ('DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8'):
+            return (self.die.cu, attr.value + self.die.cu.cu_offset)
+        elif form in ('DW_FORM_ref_addr', 'DW_FORM_ref'):
+            cualen = len(self.die.cu.dwarfinfo._unsorted_CUs)
+            i = bisect_right(self.die.cu.dwarfinfo._CU_offsets, val) - 1
+            cu = self.die.cu.dwarfinfo._unsorted_CUs[i]
+            return (cu, attr.value)
 
     # Returns (cu, die_offset) or None if not a navigable
     def ref_target(self, index):
@@ -370,15 +426,7 @@ class DIETableModel(QAbstractTableModel):
             if row >= self.meta_count:
                 attr_name = self.keys[row - self.meta_count]
                 attr = self.attributes[attr_name]
-                val = attr.value
-                form = attr.form
-                if form in ('DW_FORM_ref1', 'DW_FORM_ref2', 'DW_FORM_ref4', 'DW_FORM_ref8'):
-                    return (self.die.cu, attr.value + self.die.cu.cu_offset)
-                elif form in ('DW_FORM_ref_addr', 'DW_FORM_ref'):
-                    cualen = len(self.die.cu.dwarfinfo._unsorted_CUs)
-                    i = bisect_right(self.die.cu.dwarfinfo._CU_offsets, val) - 1
-                    cu = self.die.cu.dwarfinfo._unsorted_CUs[i]
-                    return (cu, attr.value)
+                return self.ref_target_from_attr(attr)
         except IndexError as exc:
             from .__main__ import version
             from .crash import report_crash
@@ -386,5 +434,18 @@ class DIETableModel(QAbstractTableModel):
             tb = exc.__traceback__
             report_crash(exc, tb, version, currentframe())
             return None
+        
+    def format_attr_name(self, key):
+        return key if self.prefix or not str(key).startswith('DW_AT_') else key[6:]
+    
+    def format_tag(self, tag):
+        return tag if self.prefix or not str(tag).startswith('DW_TAG_') else tag[7:]
+
+def on_details_row_dclick(index, o, win):
+    # For now, used for drilling into DWARF expressions
+    if isinstance(o, DWARFExprOp) and op_has_nested_expression(o):
+        ExpressionDlg(win, 'Nested expression', o.args[0], win.expr_formatter()).exec()
+    elif isinstance(o, Sequence) and is_parsed_expression(o[-1]):
+        ExpressionDlg(win, 'Expression', o[-1], win.expr_formatter()).exec()
 
 
