@@ -2,6 +2,7 @@ from bisect import bisect_left
 from elftools.dwarf.ranges import BaseAddressEntry as RangeBaseAddressEntry, RangeEntry
 from elftools.dwarf.locationlists import LocationExpr
 from elftools.dwarf.dwarf_expr import DWARFExprParser
+from elftools.dwarf.callframe import FDE
 
 from dwex.dwarfone import DWARFExprParserV1
 
@@ -10,7 +11,102 @@ class NoBaseError(Exception):
 
 def has_code_location(die):
     attr = die.attributes
-    return 'DW_AT_ranges' in attr or ('DW_AT_low_pc' in attr and 'DW_AT_high_pc' in attr)  
+    return 'DW_AT_ranges' in attr or ('DW_AT_low_pc' in attr and 'DW_AT_high_pc' in attr)
+
+class CodeLocationSimple:
+    def __init__(self, attr):
+        self.low = attr['DW_AT_low_pc'].value
+        self.hi = attr['DW_AT_high_pc'].value
+        if not attr['DW_AT_high_pc'].form == 'DW_FORM_addr':
+            self.hi += self.low
+
+    def start_address(self):
+        return self.low
+    
+    def in_range(self, ip):
+        return self.low <= ip < self.hi
+
+    def intersects_fde(self, fde):
+        fde_begin = fde.header.initial_location
+        fde_end = fde_begin + fde.header.address_range
+        return fde_begin < self.hi and self.low < fde_end
+
+class CodeLocationRanges:
+    def __init__(self, die):
+        cu_base = None
+        rl = get_die_ranges(die)
+        l = []
+        for r in rl:
+            if isinstance(r, RangeEntry):
+                if not r.is_absolute and cu_base is None:
+                    cu_base = get_cu_base(die) # This may throw
+                if r.is_absolute:
+                    r = (r.begin_offset, r.end_offset)
+                else:
+                    r = (cu_base + r.begin_offset, cu_base + r.end_offset)
+                l.append(r)
+            else: # Base entry
+                cu_base = r.base_address
+        self.ranges = l
+
+    def start_address(self):
+        return min(*(low for (low, hi) in self.ranges))
+    
+    def in_range(self, ip):
+        return any(1 for (low, hi) in self.ranges if low <= ip < hi)
+
+    def intersects_fde(self, fde):
+        fde_begin = fde.header.initial_location
+        fde_end = fde_begin + fde.header.address_range        
+        return any(1 for (low, hi) in self.ranges if fde_begin < hi and low < fde_end)
+
+# Returns a code location object - range or ranges
+# May throw a NoBaseError
+def get_code_location(die):
+    attr = die.attributes
+    if 'DW_AT_ranges' in attr:
+        return CodeLocationRanges(die)
+    else:
+        return CodeLocationSimple(attr)
+
+# Respects caching. None if no ranges section. Returns the raw ranges
+def get_die_ranges(die):
+    di = die.dwarfinfo
+    if not di._ranges:
+        di._ranges = di.range_lists()
+    if not di._ranges:
+        return None
+    return di._ranges.get_range_list_at_offset(die.attributes['DW_AT_ranges'].value, cu=die.cu)
+
+# Doesn't return None, returns False if not found
+def get_di_frames(di):
+    if di._frames is None:
+        has_cfi = di.has_CFI()
+        if di.has_EH_CFI():
+            if has_cfi:
+                di._frames = di.EH_CFI_entries() + di.CFI_entries()
+            else:
+                di._frames = di.EH_CFI_entries()
+        elif has_cfi:
+            di._frames = di.CFI_entries()
+        else:
+            di._frames = False
+    return di._frames
+
+def get_frame_rules_for_die(die):
+    """
+       Returns a list of dictionaries with 'pc', 'cfa' and other registers, as found in DecodedCallFrameTable.table
+       for the FDE entries that overlap with the range or ranges of the provided DIE.
+    """
+    frames = get_di_frames(die.dwarfinfo)
+    if not frames:
+        return None
+    try:
+        code_loc = get_code_location(die)
+    except NoBaseError:
+        return None
+    entries = [f for f in frames if isinstance(f, FDE) and code_loc.intersects_fde(f)]
+    return [de for e in entries for de in e.get_decoded().table]    
 
 def is_inline(func):
     return 'DW_AT_inline' in func.attributes and func.attributes['DW_AT_inline'].value != 0
@@ -27,8 +123,53 @@ def is_block(form):
 def DIE_name(die):
     return die.attributes['DW_AT_name'].value.decode('utf-8', errors='ignore')
 
+# Supports both / and \ - current system separator might not match the system the file came from
+# so os.path.basename won't do
+def strip_path(filename):
+    p = filename.rfind("/")
+    pbsl = filename.rfind("\\")
+    if pbsl >= 0 and (p < 0 or pbsl > p):
+        p = pbsl
+    return filename[p+1:] if p >= 0 else filename
+
+def top_die_file_name(die, Default = 'N/A'):
+    if DIE_has_name(die):
+        source_name = DIE_name(die)
+        return strip_path(source_name)
+    elif 'DW_AT_decl_file' in die.attributes:
+        val = die.attributes['DW_AT_decl_file'].value
+        if val > 0:
+            if die.cu._lineprogram is None:
+                die.cu._lineprogram = die.dwarfinfo.line_program_for_CU(die.cu)
+            delta = 1 if die.cu.version < 5 else 0
+            return strip_path(die.cu._lineprogram.header.file_entry[val-delta].name.decode('utf-8', errors='ignore'))
+    return Default
+
+ # See #1742
+def DIE_has_name(die):
+    """DIE object has a name attribute and the name is bytes or compatible
+    """
+    return 'DW_AT_name' in die.attributes and die.attributes['DW_AT_name'].value is not None and hasattr(die.attributes['DW_AT_name'].value, 'decode')
+
 def safe_DIE_name(die, default = ''):
-    return die.attributes['DW_AT_name'].value.decode('utf-8', errors='ignore') if 'DW_AT_name' in die.attributes else default
+    return die.attributes['DW_AT_name'].value.decode('utf-8', errors='ignore') if 'DW_AT_name' in die.attributes and die.attributes['DW_AT_name'].value is not None and hasattr(die.attributes['DW_AT_name'].value, 'decode') else default
+
+def follow_ref_if_present(die, attr_name):
+    return die.get_DIE_from_attribute(attr_name) if attr_name in die.attributes else die
+
+def subprogram_name(die, default=''):
+    """
+        Gets name of a DIE. If not available, drills back to abstract_origin, then specification
+    """
+    if 'DW_AT_name' in die.attributes:
+        return die.attributes['DW_AT_name'].value.decode('UTF-8')
+    die = follow_ref_if_present(die, 'DW_AT_abstract_origin')
+    if 'DW_AT_name' in die.attributes:
+        return die.attributes['DW_AT_name'].value.decode('UTF-8')
+    die = follow_ref_if_present(die, 'DW_AT_specification')
+    if 'DW_AT_name' in die.attributes:
+        return die.attributes['DW_AT_name'].value.decode('UTF-8')
+    return default
 
 def DIE_is_ptr_to_member_struct(type_die):
     if type_die.tag == 'DW_TAG_structure_type':
@@ -53,15 +194,11 @@ class TypeDesc(object):
 def find_cu_by_address(di, address):
     if di._aranges:
         cuoffset = di._aranges.cu_offset_at_addr(address)
-        if cuoffset is None:
-            return None
-        i = bisect_left(di._CU_offsets, cuoffset)
-        return di._unsorted_CUs[i]
-    else:
-        for cu in di._unsorted_CUs:
-            if ip_in_range(cu.get_top_DIE(), address):
+        if cuoffset is not None:
+            cu = di._unsorted_CUs[bisect_left(di._CU_offsets, cuoffset)]
+            if cu.cu_offset == cuoffset:
                 return cu
-    return None
+    return next((cu for cu in di._unsorted_CUs if ip_in_range(cu.get_top_DIE(), address)), None)
 
 # May return None or raise NoBaseError
 def get_cu_base(die):
@@ -70,14 +207,10 @@ def get_cu_base(die):
         return top_die.attributes['DW_AT_low_pc'].value
     elif 'DW_AT_entry_pc' in top_die.attributes:
         return top_die.attributes['DW_AT_entry_pc'].value
-    # TODO: ranges?
     elif 'DW_AT_ranges' in top_die.attributes:
-        di = die.dwarfinfo
-        if not di._ranges:
-            di._ranges = di.range_lists()
-        if not di._ranges: # Absent in the DWARF file
+        rl = get_die_ranges(top_die)
+        if rl is None:
             raise NoBaseError()
-        rl = di._ranges.get_range_list_at_offset(top_die.attributes['DW_AT_ranges'].value, cu=die.cu)
         base = None
         for r in rl:
             if isinstance(r, RangeEntry) and r.is_absolute and (base is None or r.begin_offset < base):
@@ -92,6 +225,7 @@ def get_cu_base(die):
 
 # Returns a list of DIEs objects for top level functions that contain the address
 # Inlines analyzed later
+# TODO: namespaces
 def find_funcs_at_address(cu, address):
     funcs = []
     top_die = cu.get_top_DIE()
@@ -110,7 +244,7 @@ def find_funcs_at_address(cu, address):
         if die.tag in ('DW_TAG_subprogram', 'DW_TAG_global_subroutine') and has_code_location(die):
             if 'DW_AT_range' in die.attributes:
                 cu_base = top_die.attributes['DW_AT_low_pc'].value
-                rl = di._ranges.get_range_list_at_offset(die.attributes['DW_AT_ranges'].value)
+                rl = di._ranges.get_range_list_at_offset(die.attributes['DW_AT_ranges'].value, cu = die.cu)
                 for r in rl:
                     if r.begin_offset <= address - cu_base < r.end_offset:
                         funcs.append(die)
@@ -126,6 +260,7 @@ def find_funcs_at_address(cu, address):
 # Find helper:
 # Returns true if the specified IP is in [low_pc, high_pc)
 # Or in ranges
+# TODO: rewrite with CodeLocation objects
 def ip_in_range(die, ip):
     if 'DW_AT_ranges' in die.attributes:
         di = die.dwarfinfo
@@ -170,20 +305,22 @@ def get_source_line(die, address):
     file_and_line = None
     prevstate = None
     for entry in lp.get_entries():
+        state = entry.state
         # We're interested in those entries where a new state is assigned
-        if entry.state is None:
-            continue
-        if entry.state.end_sequence:
-            # if the line number sequence ends, clear prevstate.
-            prevstate = None
+        if state is None:
             continue
+
         # Looking for a range of addresses in two consecutive states that
         # contain the required address.
-        if prevstate and prevstate.address <= address < entry.state.address and not file_and_line:
+        if prevstate and prevstate.address <= address < state.address and not file_and_line:
             file = (die.cu.get_top_DIE().attributes['DW_AT_name'].value if not v5 and prevstate.file == 0 else lp['file_entry'][prevstate.file + (0 if v5 else -1)].name).decode('UTF-8')
             line = prevstate.line
             file_and_line = (file, line)
-        prevstate = entry.state
+
+        if state.end_sequence:
+            prevstate = None
+        else:
+            prevstate = state
     return file_and_line
 
 # Resolves source file number in an attribute to a file name
@@ -434,3 +571,6 @@ def parse_location(loc, cu, address):
         return list((DWARFExprParser(cu.structs) if cu['version'] > 1 else DWARFExprParserV1(cu.structs)).parse_expr(loc_expr))
     else:
         return []
+
+def quote_filename(fn):
+    return f'"{fn}"' if ' ' in fn else fn
\ No newline at end of file
