@@ -1,29 +1,15 @@
 from bisect import bisect_left, bisect_right
+from typing import Union
 from PyQt6.QtCore import Qt, QAbstractItemModel, QModelIndex
-from PyQt6.QtGui import QFont, QFontInfo, QBrush
-from PyQt6.QtWidgets import QApplication, QMessageBox
-from .dwarfutil import has_code_location, safe_DIE_name
+from PyQt6.QtWidgets import QApplication
 
-# Supports both / and \ - current system separator might not match the system the file came from
-# so os.path.basename won't do
-def strip_path(filename):
-    p = filename.rfind("/")
-    pbsl = filename.rfind("\\")
-    if pbsl >= 0 and (p < 0 or pbsl > p):
-        p = pbsl
-    return filename[p+1:] if p >= 0 else filename
+from elftools.dwarf.die import DIE
+from elftools.dwarf.compileunit import CompileUnit
+
+from .fx import bold_font, blue_brush
+from .dwarfutil import DIE_has_name, DIE_name, has_code_location, safe_DIE_name, top_die_file_name
+from .dwarfone import DIEV1
 
-def top_die_file_name(die):
-    if 'DW_AT_name' in die.attributes:
-        source_name = die.attributes['DW_AT_name'].value.decode('utf-8', errors='ignore')
-        return strip_path(source_name)
-    elif 'DW_AT_decl_file' in die.attributes:
-        val = die.attributes['DW_AT_decl_file'].value
-        if val > 0:
-            if die.cu._lineprogram is None:
-                die.cu._lineprogram = die.dwarfinfo.line_program_for_CU(die.cu)
-            return strip_path(die.cu._lineprogram.header.file_entry[val-1].name.decode('utf-8', errors='ignore'))
-    return "(no name)"
 
 def cu_sort_key(cu):
     return top_die_file_name(cu.get_top_DIE()).lower()
@@ -43,7 +29,7 @@ def decorate_die(die, i):
     die._children = None
     return die
 
-def load_children(parent_die, sort):
+def load_children(parent_die: Union[DIE, DIEV1] , sort: bool): #(parent_die: Union[DIE, DIEV1] , sort: bool):
     # Load and cache child DIEs in the parent DIE, if necessary
     # Assumes the check if the DIE has children has been already performed
     if not hasattr(parent_die, "_children") or parent_die._children is None:
@@ -54,11 +40,31 @@ def load_children(parent_die, sort):
                 parent_die._children.sort(key = die_sort_key)
                 for (i, die) in enumerate(parent_die._children):
                     die._i = i
-        except KeyError as ke:
+        except KeyError as exc:
             # Catching #1516
-            QMessageBox(QMessageBox.Icon.Warning, "DWARF Explorer",
-                "This executable file is corrupt or incompatible with the current version of DWARF Explorer. Please consider creating a new issue at https://github.com/sevaa/dwex/, and share this file with the tech support.",
-                QMessageBox.StandardButton.Ok, QApplication.instance().win).show()
+            from .__main__ import version
+            from .crash import report_crash
+            from inspect import currentframe
+            tb = exc.__traceback__
+            ctxt = dict()
+            try:
+                cu = parent_die.cu
+                ctxt['cu_offset'] = cu.cu_offset
+                ctxt['dwarf_config'] = cu.dwarfinfo.config
+                abbrev_codes = set(d.abbrev_code for d in cu._dielist if not d.is_null())
+                at = cu.get_abbrev_table()
+                format_attr_in_abbrev = lambda a: (a.name, a.form, a.value) if a.value is not None else (a.name, a.form)
+                format_abbr = lambda ab: (ab.decl.tag, ab._has_children, tuple(format_attr_in_abbrev(a) for a in ab.decl.attr_spec))
+                ctxt['abbrevs'] = {c: format_abbr(at.get_abbrev(c)) for c in abbrev_codes}
+                stm = cu.dwarfinfo.debug_info_sec.stream
+                crash_pos = ctxt['crash_pos'] = stm.tell()
+                slice = stm.getbuffer()[cu.cu_offset:crash_pos+1]
+                ctxt['cu_in_info'] =  ' '.join("%02x" % b for b in slice)
+            except Exception:
+                pass
+            report_crash(exc, tb, version, currentframe(), ctxt)
+
+            QApplication.instance().win.show_warning("This executable file is corrupt or incompatible with the current version of DWARF Explorer. Please consider creating a new issue at https://github.com/sevaa/dwex/, and share this file with the tech support.")
             parent_die._children = []
 
 class DWARFTreeModel(QAbstractItemModel):
@@ -67,9 +73,6 @@ class DWARFTreeModel(QAbstractItemModel):
         self.prefix = prefix
         self.top_dies = [decorate_die(CU.get_top_DIE(), i) for (i, CU) in enumerate(di._CUs)]
         self.highlight_condition = None
-        fi = QFontInfo(QApplication.font())
-        self.bold_font = QFont(fi.family(), fi.pointSize(), QFont.Weight.Bold)
-        self.blue_brush = QBrush(Qt.GlobalColor.blue)
         self.sortcus = sortcus
         self.sortdies = sortdies
 
@@ -126,16 +129,16 @@ class DWARFTreeModel(QAbstractItemModel):
                     s = ('DW_TAG_user_%X' if self.prefix else 'user_%X') % die.tag
                 else:
                     s = die.tag if self.prefix or not str(die.tag).startswith('DW_TAG_') else die.tag[7:]
-                if 'DW_AT_name' in die.attributes:
-                    s += ": " + die.attributes['DW_AT_name'].value.decode('utf-8', errors='ignore')
+                if DIE_has_name(die):
+                    s += ": " + DIE_name(die)
                 return s
         elif role == Qt.ItemDataRole.ToolTipRole:
             if die.tag == 'DW_TAG_compile_unit' or die.tag == 'DW_TAG_partial_unit':
                 return safe_DIE_name(die, None)
         elif role == Qt.ItemDataRole.ForegroundRole and self.is_highlighted(die):
-            return self.blue_brush
+            return blue_brush
         elif role == Qt.ItemDataRole.FontRole and self.is_highlighted(die):
-            return self.bold_font
+            return bold_font()
 
     # The rest is not Qt callbacks
 
@@ -157,7 +160,8 @@ class DWARFTreeModel(QAbstractItemModel):
 
     def clear_highlight(self):
         self.highlight_condition = None
-        self.dataChanged.emit(self.createIndex(0, 0, self.top_dies[0]), self.createIndex(len(self.top_dies)-1, 0, self.top_dies[-1]), (Qt.ItemDataRole.ForegroundRole, Qt.ItemDataRole.FontRole))
+        if len(self.top_dies):
+            self.dataChanged.emit(self.createIndex(0, 0, self.top_dies[0]), self.createIndex(len(self.top_dies)-1, 0, self.top_dies[-1]), (Qt.ItemDataRole.ForegroundRole, Qt.ItemDataRole.FontRole))
 
     def has_highlight(self, key):
         return bool(self.highlight_condition and key in self.highlight_condition)
@@ -272,12 +276,37 @@ class DWARFTreeModel(QAbstractItemModel):
             cu_offset = cu.cu_offset
             # Parse all DIEs in the current CU
             if cu_cond(cu) if cu_cond else True:
-                for die in cu.iter_DIEs():
-                    # Quit condition with search from position - quit once we go past the starting position after the wrap
-                    if have_start_pos and cu_offset >= start_cu_offset and die.offset > start_die_offset and wrapped:
-                        break
-                    if not die.is_null() and (not have_start_pos or cu_offset != start_cu_offset or (not wrapped and die.offset > start_die_offset)) and cond(die):
-                        return self.index_for_die(die)
+                try: #1516
+                    for die in cu.iter_DIEs():
+                        # Quit condition with search from position - quit once we go past the starting position after the wrap
+                        if have_start_pos and cu_offset >= start_cu_offset and die.offset > start_die_offset and wrapped:
+                            break
+                        if not die.is_null() and (not have_start_pos or cu_offset != start_cu_offset or (not wrapped and die.offset > start_die_offset)) and cond(die):
+                            return self.index_for_die(die)
+                except KeyError as exc: #1516
+                    from .__main__ import version
+                    from .crash import report_crash
+                    from inspect import currentframe
+                    tb = exc.__traceback__
+                    ctxt = dict()
+                    try:
+                        ctxt['cu_offset'] = cu.cu_offset
+                        ctxt['dwarf_config'] = cu.dwarfinfo.config
+                        abbrev_codes = set(d.abbrev_code for d in cu._dielist if not d.is_null())
+                        at = cu.get_abbrev_table()
+                        format_attr_in_abbrev = lambda a: (a.name, a.form, a.value) if a.value is not None else (a.name, a.form)
+                        format_abbr = lambda ab: (ab.decl.tag, ab._has_children, tuple(format_attr_in_abbrev(a) for a in ab.decl.attr_spec))
+                        ctxt['abbrevs'] = {c: format_abbr(at.get_abbrev(c)) for c in abbrev_codes}
+                        stm = cu.dwarfinfo.debug_info_sec.stream
+                        crash_pos = ctxt['crash_pos'] = stm.tell()
+                        slice = stm.getbuffer()[cu.cu_offset:crash_pos+1]
+                        ctxt['cu_in_info'] =  ' '.join("%02x" % b for b in slice)
+                    except Exception:
+                        pass
+                    report_crash(exc, tb, version, currentframe(), ctxt)
+
+                    QApplication.instance().win.show_warning("This executable file is corrupt or incompatible with the current version of DWARF Explorer. Please consider creating a new issue at https://github.com/sevaa/dwex/issues, and share this file with the tech support.")
+                    return False
 
             # We're at the end of the CU. What next?
             if cu._i < len(self.top_dies) - 1: # More CUs to scan
