@@ -1,6 +1,6 @@
 import os
 from PyQt6.QtCore import Qt, QRectF, QSizeF, QPointF, QByteArray
-from PyQt6.QtGui import QKeySequence, QAction, QImage, QPixmap, QPainter, QIcon
+from PyQt6.QtGui import QKeySequence, QAction, QImage, QPixmap, QPainter, QIcon, QFont
 from PyQt6.QtWidgets import *
 from PyQt6.QtSvg import QSvgRenderer
 
@@ -16,6 +16,9 @@ def setup_menu(win):
     win.savesection_menuitem = file_menu.addAction("Save a section as...")
     win.savesection_menuitem.triggered.connect(win.on_savesection)
     win.savesection_menuitem.setEnabled(False)
+    win.loadexec_menuitem = file_menu.addAction("Load companion executable...")
+    win.loadexec_menuitem.triggered.connect(win.on_loadexec)
+    win.loadexec_menuitem.setEnabled(False)
     win.mru_menu = file_menu.addMenu("Recent files")
     if len(win.mru):
         win.populate_mru_menu()
@@ -74,6 +77,9 @@ def setup_menu(win):
     win.cuproperties_menuitem = view_menu.addAction("CU properties...")
     win.cuproperties_menuitem.setEnabled(False)
     win.cuproperties_menuitem.triggered.connect(win.on_cuproperties)
+    view_menu.addSeparator()
+    theme_menuitem = view_menu.addAction("Theme...")
+    theme_menuitem.triggered.connect(win.on_changetheme)
     #########
     edit_menu = menu.addMenu("Edit")
     win.copy_menuitem = edit_menu.addAction("Copy value")
@@ -130,6 +136,19 @@ def setup_menu(win):
     win.localsat_menuitem = ana_menu.addAction("Locals at address...")
     win.localsat_menuitem.setEnabled(False)
     win.localsat_menuitem.triggered.connect(win.on_localsat)
+    win.funcmap_menuitem = ana_menu.addAction("Function map...")
+    win.funcmap_menuitem.setEnabled(False)
+    win.funcmap_menuitem.triggered.connect(win.on_funcmap)
+    ana_menu.addSeparator()
+    win.aranges_menuitem = ana_menu.addAction("Aranges...")
+    win.aranges_menuitem.setEnabled(False)
+    win.aranges_menuitem.triggered.connect(win.on_aranges)
+    win.frames_menuitem = ana_menu.addAction("Frames...")
+    win.frames_menuitem.setEnabled(False)
+    win.frames_menuitem.triggered.connect(win.on_frames)
+    win.unwind_menuitem = ana_menu.addAction("Unwind info...")
+    win.unwind_menuitem.setEnabled(False)
+    win.unwind_menuitem.triggered.connect(win.on_unwind)
     ########
     help_menu = menu.addMenu("Help")
     about_menuitem = help_menu.addAction("About...")
@@ -218,10 +237,7 @@ def setup_toolbar(win):
     win.prevhl_tbitem.triggered.connect(win.on_prevhl)
     win.prevhl_tbitem.setToolTip("Navigate to the previous highlighted DIE in the tree")
 
-
-def setup_ui(win):
-    setup_menu(win)
-    setup_toolbar(win)
+def setup_explorer(win):
     # Set up the left pane and the right pane
     tree = win.the_tree = QTreeView()
     tree.header().hide()
@@ -230,6 +246,7 @@ def setup_ui(win):
     rpane = QSplitter(Qt.Orientation.Vertical)
     die_table = win.die_table = QTableView()
     die_table.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
+    die_table.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
     die_table.doubleClicked.connect(win.on_attribute_dclick)
     rpane.addWidget(die_table)
     
@@ -240,6 +257,8 @@ def setup_ui(win):
     rbpane.addWidget(details_warning)
     details_table = win.details_table = QTableView()
     details_table.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
+    details_table.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
+    details_table.doubleClicked.connect(win.on_details_dclick)
     rbpane.addWidget(details_table)
     rbp = QWidget()
     rbp.setLayout(rbpane)
@@ -256,5 +275,20 @@ def setup_ui(win):
     spl.setStretchFactor(1, 1) 
     win.setCentralWidget(spl)
 
+def setup_splash(win):
+    l = QLabel()
+    l.setText("Drag\na file\nhere")
+    l.setAlignment(Qt.AlignmentFlag.AlignCenter)
+    f = QFont("Verdana", 50)
+    f.setStyleHint(QFont.StyleHint.SansSerif)
+    l.setFont(f)
+    l.setEnabled(False)
+    win.setCentralWidget(l)
+
+def setup_ui(win):
+    setup_menu(win)
+    setup_toolbar(win)
+    setup_splash(win)
+
     win.setWindowTitle("DWARF Explorer")
     win.resize(win.font_metrics.averageCharWidth() * 250, win.font_metrics.height() * 60)
