@@ -28,7 +28,7 @@ def test_CU_iter(di):
 #test_tree_for("\\\\mac\\seva\\quincy", dump_compiler)
 
 #test_tree("E:\\Seva\\Projects\\pyelftools\\test\\testfiles_for_readelf")
-#test_tree("E:\\Seva\\Projects\\pyelftools\\test\\testfiles_for_dwarfdump")
+test_tree("E:\\Seva\\Projects\\pyelftools\\test\\testfiles_for_dwarfdump")
 test_tree("samples\\other")
 
 def f(di):
